function capacity_simulation
clear all; 
close all;
warning off MATLAB:log:LogOfZero
warning off MATLAB:divideByZero
global_variables_initialize;

channel_type = 1;          
%1=mobile, 2=fixed
                           
NPath = 4;              %number of reflected paths
NMeasurement = 1000;    %Number of measurements to be simulated
NOFDM = 256;

ignoreFading = 0;
    %if ignoreFading=1, the fading is omitted (used for AWGN channel).

OFDM_subcarrier_locations = OFDM_get_subcarriers( NOFDM );          
    %OFDM symbol with amplitude 1 on datasubcarriers and 2 on pilots.

remove_pilots_indexes = find(OFDM_subcarrier_locations == 2);
OFDM_data_subcarriers_locations = OFDM_subcarrier_locations;

OFDM_data_subcarriers_locations(remove_pilots_indexes) = 0;         
    %setting pilots to zero for capacity calculations

len = length(OFDM_subcarrier_locations);
Used_frequency_indexes = ((ceil(len/2)-(NOFDM/2)+1):(ceil(len/2)+(NOFDM/2)));

%SNR levels
   BPSK_0_5 = 3;
   QPSK_0_5 = 6;
   QPSK_0_75 = 9;
   QAM_16_0_5 = 12;
   QAM_16_0_75 = 15;
   QAM_64_0_5 = 18;
   QAM_64_0_66 = 20;
   QAM_64_0_75 = 21;
   QAM_64_0_83 = 23;

%Link Budget Parameters
  %TRANSMITTER
  Transmit_Power_Per_Element = 10; %dBw
  Transmit_Antenna_Combination_Gain = 3; %dB, for 2 transmit antennas
  Transmit_Antenna_Gain = 12; %dB
  Pilot_Power_Boosting_Gain = -0.7; 
      %dB,loss of pilot power(5/6 of subcarriers used for data)
  EIRP = Transmit_Power_Per_Element+Transmit_Antenna_Combination_Gain+Transmit_Antenna_Gain+Pilot_Power_Boosting_Gain;
  disp(' ')
  disp('LINK BUDGET PARAMETERS:')
  disp(['EIRP=' num2str(EIRP)])
  %RECEIVER
  Receiver_Antenna_Gain = 0;  %dB, 0 for Mobile, 10 etc for fixed.
  Receiver_Antenna_Diversity_Gain = 0; %3dB for 2 antennas (fixed)
  Receiver_Noise_Figure = 6;
  RECEIVER_GAIN = Receiver_Antenna_Gain+Receiver_Antenna_Diversity_Gain-Receiver_Noise_Figure;
  disp(['Receiver Gain=' num2str(RECEIVER_GAIN)])
  %NOISE
  Noise_Thermal_Density = -194;   %dBw/Hz
  global Fs_OFDM;                 %BW of OFDM symbol
  TOTAL_TERMAL_NOISE = Noise_Thermal_Density + 10*log10(Fs_OFDM);
  %MARGINS
  Interference_Margin = 2;
  Penetration_Loss = 0;       %10 for indoor environments
  MARGINS = Interference_Margin+Penetration_Loss;
  disp(['Margins=' num2str(MARGINS)])
  %PATH LOSS
  PATH_LOSS = 140;
  disp(['Path Loss=' num2str(PATH_LOSS)])
  disp(['Noise=' num2str(TOTAL_TERMAL_NOISE)])
  
  SNR_NO_FADING = (EIRP+RECEIVER_GAIN-MARGINS-PATH_LOSS)-TOTAL_TERMAL_NOISE;
  disp(['SNR WITHOUT FADING=' num2str(SNR_NO_FADING)])
  disp(['LOWER SNR LIMIT FOR BPSK=' num2str(BPSK_0_5)])
  SNR_MARGIN = SNR_NO_FADING - BPSK_0_5;
  disp('-------------------------')
  disp(['SNR MARGIN=' num2str(SNR_MARGIN)])
  disp('-------------------------')
%Finished Link Budget parameters. Link budget considered with
%1000 measurements, which then resulted in a BER of 0.88%
    

H_f_array = zeros(NMeasurement,length(OFDM_subcarrier_locations));
Capacities = zeros(1,NMeasurement);
maxlevels = 0;
maxcapacity = 0;
mincapacity = inf;
minlevels = inf;
sum_Hf = zeros(1,NMeasurement);
SNR_too_low_counter = 0;
[dummy1 doppfilt dummy2] = doppler_filter(1); %used for selecting random fading coeff
[dummy1 gaussdoppfilt dummy2] = gaussian_doppler_filter(1); %used for selecting random fading coeff

for n=1:NMeasurement
   
   disp(' ')
   disp('**************')
   disp(['MEASUREMENT NUMBER:' num2str(n)])
   disp('**************')
   [h g_t h_g_t h_g_t_ideal] = channel_filter_cascade(NPath);   
   h = h_g_t;
   h_taps = find(abs(h) > (1/log(10+128+1)));
        %consider components as distinc paths if amplitude exceeds minimum
        %attenuation for a path.
   for k=1:length(h_taps)
     tap = h(h_taps(k));
     if(channel_type==1)   
       %mobile 
       if(k==1 || h_taps(k)-h_taps(k-1)>2) %used to distinguish paths from sidelobes
         fadingcoeff=doppfilt(ceil(rand*(length(doppfilt))))+j*doppfilt(ceil(rand*(length(doppfilt))));               
       end
       tap = tap*fadingcoeff;
     elseif(channel_type == 2)               
       %fixed                                                                                                   
       if(k==1 || h_taps(k)-h_taps(k-1)>2) %used to distinguish paths from sidelobes
         fadingcoeff=gaussdoppfilt(ceil(rand*(length(gaussdoppfilt))))+j*gaussdoppfilt(ceil(rand*(length(gaussdoppfilt))));                                          
       end
       tap = tap*fadingcoeff;
     end
     if(ignoreFading==1)
         tap = tap/fadingcoeff;
     end
     h(h_taps(k))=tap; 
        %update according to fading
   end
   
   H_f_array(n,:) = fftshift(fft(h,length(OFDM_subcarrier_locations))); 
   sum_Hf(n) = sum(H_f_array(n,:));
   
   received_levels=20*log10(abs(H_f_array(n,:).*OFDM_data_subcarriers_locations));
  
   receivedSNR = zeros(length(Used_frequency_indexes));
   Capacity = 0;
   
   k=1;
   subclength = length(OFDM_subcarrier_locations);
   for l=((ceil(subclength/2)-(NOFDM/2)+1):(ceil(subclength/2)+(NOFDM/2)))
       receivedSNR(k) = received_levels(l)+SNR_NO_FADING;

       if (BPSK_0_5<=receivedSNR(k) && receivedSNR(k)<QPSK_0_5)
           Capacity = Capacity + 1*0.5;    
            %capacity of subcarrier n with BPSK 1/2 rate 
       elseif (QPSK_0_5<=receivedSNR(k) && receivedSNR(k)<QPSK_0_75)
           Capacity = Capacity + 2*0.5;    
            %capacity of subcarrier n with QPSK 1/2 rate 
       elseif (QPSK_0_75<=receivedSNR(k) && receivedSNR(k)<QAM_16_0_5)
           Capacity = Capacity + 2*0.75;   
            %capacity of subcarrier n with QPSK 3/4 rate
       elseif (QAM_16_0_5<=receivedSNR(k) && receivedSNR(k)<QAM_16_0_75)
           Capacity = Capacity + 4*0.5;    
            %capacity of subcarrier n with 16QAM 1/2 rate
       elseif (QAM_16_0_75<=receivedSNR(k) && receivedSNR(k)<QAM_64_0_5)
           Capacity = Capacity + 4*0.75;   
            %capacity of subcarrier n with 16QAM 3/4 rate
       elseif (QAM_64_0_5<=receivedSNR(k) && receivedSNR(k)<QAM_64_0_66)
           Capacity = Capacity + 6*0.5;    
            %capacity of subcarrier n with 64QAM 1/2 rate
       elseif (QAM_64_0_66<=receivedSNR(k) && receivedSNR(k)<QAM_64_0_75)
           Capacity = Capacity + 6*(2/3);  
            %capacity of subcarrier n with 64QAM 2/3 rate
       elseif (QAM_64_0_75<=receivedSNR(k) && receivedSNR(k)<QAM_64_0_83)
           Capacity = Capacity + 6*(3/4);  
            %capacity of subcarrier n with 64QAM 3/4 rate
       elseif (QAM_64_0_83<=receivedSNR(k))
           Capacity = Capacity + 6*(5/6);  
            %capacity of subcarrier n with 64QAM 5/6 rate
       else
           Capacity = Capacity + 0;        %too low SNR
           if(OFDM_data_subcarriers_locations(l)>0)
              SNR_too_low_counter = SNR_too_low_counter + 1;
                %only data carriers considered
           end
       end
       k = k+1;
   end


   Symbol_duration = 102.9*10^-6;
   Capacities(1,n) = Capacity/Symbol_duration;

   if (Capacities(1,n)>maxcapacity)
            maxlevels = received_levels+SNR_NO_FADING;
            maxcapacity = Capacities(1,n);
   end
   if (Capacities(1,n)<mincapacity)
            minlevels = received_levels+SNR_NO_FADING;
            mincapacity = Capacities(1,n);
   end

end


figure(1);
subplot(3,1,1);
f = [((-1/2):(1/length(H_f_array(NMeasurement,:))):((1/2)-1/length(H_f_array(NMeasurement,:))))];
plot(f,20*log10(abs(H_f_array(NMeasurement,:))))
title('Response of channel and system filter cascade last measurement')
ylabel('Frequency Response [dB]')
xlabel('Frequencies. -50MHz --> +50MHz')
subplot(3,1,2);
index = ((ceil(len/2)-(NOFDM/2)+1):(ceil(len/2)+(NOFDM/2)));
stem(OFDM_subcarrier_locations(index));
title('transmitted ofdm symbol (simplified) for capacity estimations')
xlabel('subcarrier number')
ylabel('amplitude')
subplot(3,1,3)
stem(received_levels(index))
global Fs_OFDM;
title(['Response of channel and system filter cascade at last measurement over OFDM BW=' num2str(Fs_OFDM)])
xlabel('subcarrier number')
ylabel('level [dB]')

figure(2);
subplot(2,1,1)
stem(maxlevels(index))
title(['received levels at maximum capacity of:' num2str(floor(maxcapacity)) 'bits/sec'])
ylabel('amplitude [dB]')
xlabel('subcarrier number')
subplot(2,1,2)
stem(minlevels(index))
title(['received levels at minimum capacity of:' num2str(floor(mincapacity)) 'bits/sec'])
ylabel('amplitude [dB]')
xlabel('subcarrier number')

global data_carriers;
disp(' ')
disp('CAPACITY METRICS:')
avg = mean(Capacities);
sigma = std(Capacities);
disp(['Mean value:' num2str(avg) 'bit/s']);
disp(['Standard deviation:' num2str(sigma) 'bit/s']);
disp(['Maximum measured capacity:' num2str(maxcapacity) 'bit/s']);
disp(['Minimum measured capacity:' num2str(mincapacity) 'bit/s']);
disp(['Maximum theoretical capacity(QAM64 5/6 rate over all carriers):' num2str(data_carriers*6*(5/6)/Symbol_duration) 'bit/s'])
disp(' ');

figure(3);
cap_min = min(Capacities);
cap_max = max(Capacities);
range = cap_min:(cap_max-cap_min)/(length(Capacities)-1):cap_max;
cap_sorted = sort(Capacities);
cap_cdf = zeros(1,length(range));
for n=1:length(range)
    cap_cdf(n) = length(find(cap_sorted<range(n)));
end
cap_cdf = cap_cdf./length(cap_cdf);
plot(range,cap_cdf)
title('Empirical Capacity CDF')
xlabel('Capacity (bit/s)')
ylabel('Probability P(c<=C)')
%subplot(2,1,2)
figure(4);
hist(Capacities,20);
title(['Capacity histogram of ' num2str(NMeasurement) ' Measurements'])
xlabel('Capacity bins (bit/s)')
ylabel('Number of occurences')

figure(5)
cdfplot(Capacities);

range_subcarriers=[((ceil(subclength/2)-(NOFDM/2)+1):(ceil(subclength/2)+(NOFDM/2)))];
SNR_too_low_data_carriers_avg = SNR_too_low_counter/NMeasurement;
num_of_data_carriers = length(find(OFDM_data_subcarriers_locations>0));
disp(['Number of data carriers:' num2str(num_of_data_carriers)])
disp(['Number of data carriers below 3dB averaged over ' num2str(NMeasurement) ' measurements is:' num2str(SNR_too_low_data_carriers_avg)])
disp(['This gives an outage probability of ' num2str(SNR_too_low_data_carriers_avg*100/num_of_data_carriers) '%'])

end