function [h_t_ideal g_t h_g_t h_g_t_ideal] = channel_filter_cascade(NPath)
global Fc;
global Chirp_Ts;
global Chirp_duration;
global H_f_interpol_factor;
global Chirp_oversampling_factor;
oversampling = Chirp_oversampling_factor;
resolution = Chirp_Ts;
duration = Chirp_duration;
num_of_chirp_increments = duration/resolution;

%------------------------------------------------
%STARTING WITH THE CHANNEL IMPULSE RESPONSE: h(t)
%------------------------------------------------
%the delays are given in relative format, i.e. delay = 
%relative delay. The relative delay of the first arriving component is zero
relative_delays = zeros(1,NPath);
disp(' ')
disp('DELAY RELATIVE TO duration:')
disp(['   relative delay of the first arriving component is 0'])
for n=2:NPath
 relative_delays(n) = (1/num_of_chirp_increments)*(min(((abs(rand)*num_of_chirp_increments/16)+1),(num_of_chirp_increments/16)));             
    %delay between 1 and num_of_chirp_increments/16 for chirp durations of 1024/2048 Ts   
    %delay of 1=3m path difference, while 64=192m path difference. OK for
    %indoor short range measurements.
end                                                                                                              
                                                                                                              
relative_delays = sort(relative_delays);
for n=2:NPath
    disp(['   relative delay of reflected component ' num2str(n) ' is ' num2str(relative_delays(n))]);
end
disp(' ')

%the phase differences of the reflected components are relative to the 
%first arriving component. The phase is uniformly distributed in [0,2*pi]
relative_phase_diff = zeros(1,NPath);
disp('RELATIVE PHASE DIFFERENCE OF FIRST AND SUBSEQUENT REFLECTED COMPONENTS:')
disp(['   phase of the first arriving component is 0'])
for n=2:NPath
 relative_phase_diff(n) = rand*2*pi;%(round(rand*10))*2*pi/10;
 disp(['   phase of reflected component ' num2str(n) ' is ' num2str(relative_phase_diff(n))])
end
disp(' ')


a = ones(1,NPath);
disp('RELATIVE ATTENUATION:')
disp(['   relative attenuation, a0, of the first arriving component is ' num2str(a(1))])

for n=2:NPath
    a(n) = 1/(log10((10+num_of_chirp_increments*(relative_delays(n)))));         
        %log-normal shadowing and path loss only. Fading is included in
        %simualtions.
end
a = sort(a,'descend');
for n=2:NPath
    disp(['   attenuation of reflected component ' num2str(n) ' is ' num2str(a(n))])
end

global Channel_impulse_response_time_resolution;
global Chirp_onesided_BW;
global Fc;
global Chirp_Fs;


h_g = zeros(1,num_of_chirp_increments); %for arbitrary delays.. convolution of h and g resampled
                                        %this is used for simulation where fraction delays are allowed
h_g_ideal = zeros(1,num_of_chirp_increments); %for delays of order n*Chirp_Ts (ideal delays)
h_ideal = zeros(1,num_of_chirp_increments); %ideal delay impulse response, used for illustration purposes.
freq_resolution = Chirp_duration/Chirp_Ts; %Chirp_Fs/1000000;
H = zeros(1,freq_resolution);



temp = zeros(1,NPath);
n=1;
f_lo = -(Chirp_Fs/2)/1000000;
f_hi = (Chirp_Fs/2)/1000000;
f = f_lo;
freq_incr = (f_hi-f_lo)/freq_resolution;
global Approximated_system_filter_onesided_BW;
Ts = 1/Approximated_system_filter_onesided_BW;
for t=0:resolution:duration-resolution
  for k=1:NPath
    %h_g and h_g_ideal: convolution of system filter (cos roll off)
    %and channel impulse response. "ideal" means only integer 
    %number of Ts delay in channel impulse response, whereas h_g 
    %includes arbitrary delay. The term "0*relative_delays(k)" is due to
    %baseband considerations.
    h_g(n) = h_g(n)+(a(k)/a(1))*cos_roll_off(t-4*Ts-relative_delays(k)*duration)*exp(j*(relative_phase_diff(k)-0*relative_delays(k)*duration)); 
    h_g_ideal(n) = h_g_ideal(n)+(a(k)/a(1))*cos_roll_off(t-4*Ts-(ceil(relative_delays(k)*num_of_chirp_increments)/num_of_chirp_increments)*duration)*exp(j*(relative_phase_diff(k)-0*relative_delays(k)*duration)); 
    h_ideal(n) = h_ideal(n)+(a(k)/a(1))*dirac_discrete(t-(ceil(relative_delays(k)*num_of_chirp_increments)/num_of_chirp_increments)*duration)*exp(j*(relative_phase_diff(k)-0*relative_delays(k)*duration)); 
  end
n=n+1;
end
    
n=1;
global system_filter_cutoff_factor;
for t = -system_filter_cutoff_factor*Ts:(Ts/system_filter_cutoff_factor):(system_filter_cutoff_factor*Ts-(Ts/system_filter_cutoff_factor))
    g(n) = cos_roll_off(t);
    n=n+1;
end

g_t = g;
h_t_ideal = h_ideal;
h_g_t = h_g;
h_g_t_ideal = h_g_ideal;
end