function [doppler_filter gauss_filtered gauss_sequence]   = gaussian_doppler_filter(NPath)
%gaussian_doppler_filter calculates doppler coefficients for the N Paths in
%a fixed "mobile" communication environment, i.e. only the reflectors are moving.
%NPath is the number of paths in the multipath environment

global Fd;                                    %max doppler shift (doppler frequency)
global Chirp_Ts;
global Fast_Fading_Ts_factor;

f = -Fd+1:1:Fd-1;                             %frequencies (length 2*Fm-1), end values omitted
                                              %due to infinite-valued doppler spectrum.

Sd = zeros(NPath,length(f));
sigma = Fd/3;                                 %cutoff at 3*sigma
for k=1:NPath
    for i=1:1:length(f)
    Sd(k,i) = exp(-((i-Fd)^2)/(2*(sigma^2))); %gaussian doppler spectrum
    end
end

Doppler_BW = 2*Fd;
global Gauss_Oversampling_factor;
   %gauss symbol should have large oversampling factor compared to
   %the Doppler_BW, i.e. a long sequence of correlated gauss samples
   %are produced in the filtering process.
gauss_sekv_BW = Doppler_BW*Gauss_Oversampling_factor;
gauss_sekv_duration = 1/gauss_sekv_BW;

Sd1 = zeros(NPath,gauss_sekv_BW);
gauss_re = randn(NPath,gauss_sekv_BW);    %create vector of complex gaussian values, real part.
gauss_im = randn(NPath,gauss_sekv_BW);    %create vector of complex gaussian values, imaginary part.
avg = zeros(1,NPath);
f = 1:gauss_sekv_BW;
for k=1:NPath
    for n=1:(length(Sd(k,:)))                             
        Sd1(k,((length(Sd1(k,:))/2)-Fd)+n) = Sd(k,n);
    end
    gauss_freq(k,:) = (1/Gauss_Oversampling_factor)*fft((gauss_re(k,:)+j*gauss_im(k,:)), gauss_sekv_BW);
    Gauss_Filt(k,:)    = gauss_freq(k,:).*Sd1(k,:);
    Gauss_Filt_time_domain(k,:) = ifft(Gauss_Filt(k,:));
    avg(k) = sum(abs(Gauss_Filt_time_domain(k,:)))/length(Gauss_Filt_time_domain(k,:));
end
t = 0:(length(Sd1(1,:))-1);
%the avarage amplitude values of Gauss_filt_time_domain must equal 1.
%i.e. the scale factor on the taps must produce an output with avarage
%amplitud value of unity.
for k=1:NPath
    Gauss_Filt_time_domain(k,:) = Gauss_Filt_time_domain(k,:)./avg(k);
end
doppler_filter = Sd;
gauss_filtered = Gauss_Filt_time_domain;
gauss_sequence = gauss_re + j.*gauss_im;

end
