function measurement_simulation
clear all; 
close all;
global_variables_initialize;
NPath = 6;                      %number of reflected paths
comtype = 1;                    %1=mobile , 2=fixed
if comtype == 1
 [doppler_filter_f doppler_seq_filtered gauss_sequence] = doppler_filter(NPath);
elseif comtype == 2
 [doppler_filter_f doppler_seq_filtered gauss_sequence] = gaussian_doppler_filter(NPath);
end
[h g_t h_g_t h_g_t_ideal] = channel_filter_cascade(NPath);      
    %h=ideal channel response, g_t=raised cosine filter, 
    %h_g_t=convolution of non-ideal channel response(fraction delay) 
    %and raised cosine, h_g_t_ideal=convolution of ideal channel 
    %response and raised cosine.
H_f = fft(h);
H_f_shifted = fftshift(H_f);
s_t = chirp_signal_generation;
s_t_inv = chirp_signal_generation_inverse;
autocorr_chirp = conv(s_t,s_t_inv);
S_t_freq = fft(s_t);
S_t_freq_shifted = fftshift(S_t_freq);
S_t_inv_freq = fft(s_t_inv);
S_t_inv_freq_shifted = fftshift(S_t_inv_freq);
G_t_freq = fft(g_t);
G_t_freq_shifted = (fftshift(G_t_freq));
[noise_samples noise_level] = generate_noise(0,0,NPath);

figure(1);
global Channel_impulse_response_time_resolution;
global Chirp_Ts;
global Chirp_Fs;
global Channel_impulse_response_freq_resolution;
global Chirp_onesided_BW;
global Chirp_duration;
global H_f_interpol_factor;
t=0:Chirp_Ts:Chirp_duration-Chirp_Ts;
subplot(2,1,1)
stem(t,abs(h))
xlabel('time (0 --> Chirpduration)')
ylabel('channel impulse response |h(t)|')
subplot(2,1,2)
f = -Chirp_Fs/2:(Chirp_Fs/length(H_f)):((Chirp_Fs/2)-(Chirp_Fs/length(H_f)));
plot(f,20*log10(abs(H_f_shifted)));
xlabel(['frequency ( -' num2str(Chirp_Fs/2) '[Hz] -->+' num2str(Chirp_Fs/2) '[Hz] )'])
ylabel('frequency response |H(f)| [dB]')
    
figure(2)
global Chirp_duration;
t = 0:Chirp_Ts:(Chirp_duration-Chirp_Ts);
subplot(2,1,1)
plot(t,real(s_t));
ylabel('real part of chirp s(t)')                 %no need plotting imag part
xlabel(['time (0 --> ' num2str(Chirp_duration/Chirp_Ts) '*Ts'])
subplot(2,1,2)
n = -length(autocorr_chirp)/2:1:length(autocorr_chirp)/2-1;
plot(n,(1/(Chirp_duration/Chirp_Ts))*abs(autocorr_chirp));
xlabel('nTs')
ylabel('autocorrelation function |Rss(nTs)|')
    
figure(3)
f = 0:(Chirp_Fs/length(t)):(Chirp_Fs-(Chirp_Fs/length(t)));
plot(f,20*log10(((S_t_freq_shifted))))
title('Frequency response of chirp (fft of real+j*imag of chirp)')
ylabel('S(f)[dB]')
xlabel('frequency')

figure(4)
subplot(2,1,1)
global Approximated_system_filter_onesided_BW;
global system_filter_cutoff_factor;
T = 1/(2*Approximated_system_filter_onesided_BW);
t = -system_filter_cutoff_factor:(2*system_filter_cutoff_factor/length(g_t)):(system_filter_cutoff_factor-(2*system_filter_cutoff_factor/length(g_t)));
plot(t,g_t);
title('Cos-Roll Off System Filter.')
xlabel(['time. zero-crossings at n*T=n*(1/2*30MHz)'])
ylabel('g(t), square window')
subplot(2,1,2)
sysBW = Approximated_system_filter_onesided_BW;
f = -sysBW*system_filter_cutoff_factor:(sysBW*2*system_filter_cutoff_factor/length(G_t_freq_shifted)):((sysBW*system_filter_cutoff_factor)-(sysBW*system_filter_cutoff_factor/length(G_t_freq_shifted)));
handler1=plot(f,(abs(G_t_freq_shifted))); hold on;
handler2=stem([-Approximated_system_filter_onesided_BW Approximated_system_filter_onesided_BW],[max(abs(G_t_freq_shifted)) max(abs(G_t_freq_shifted))], 'r');
legend([handler1 handler2],['filter response'],['3dB width']);
xlabel(['frequency. One Sided BW=1/2T=30MHz (at 3dB width).'])
ylabel('frequency response |G(f)|')

figure(5);
f = interp((f.*0.5),system_filter_cutoff_factor*2);
t=0:Chirp_Ts:Chirp_duration-Chirp_Ts;
subplot(2,1,1)
plot(t,abs(h_g_t));
title('convolution of raised cosine system filter and channel impulse response (no fading)')
xlabel('time')
ylabel('amplitude (abs)')
G_H = fftshift(fft(h_g_t));
subplot(2,1,2)
plot(f,abs(G_H))
title('frequency response of system filter and channel impulse response')
xlabel('baseband frequency')
ylabel('|H(f)G(f)|')


figure(6);
global Fast_Fading_Ts_factor;
global Fd; 
subplot(3,1,1)
f = -Fd+1:1:Fd-1; 
plot(f,abs(doppler_filter_f(1,:)));
title('Doppler spectrum (linear scale) of first tap')
xlabel('doppler frequency');
subplot(3,1,2)
plot(abs(gauss_sequence(1,:)));
title('gauss sample components (first tap: X_0_I + j*X_0_Q)');
ylabel('abs(real+j*imag))');
xlabel('sample number')
subplot(3,1,3)
plot(abs(doppler_seq_filtered(1,:)));
title('doppler filtered gauss samples (first tap)')
ylabel('abs(real+j*imag)');
xlabel('sample number');
        
        
%------------------------------------------    
%STARTING SIMULATION CODE.
%------------------------------------------
%A repetitive chirp signal is sent.  Each chirp signal lasts 2048*Ts.
%Ts equals 10ns. 2048*Ts= 20.48microsec. The simulation period is set
%to 100*Chirp_duration=204800*Ts
%The length of the doppler coefficients is 2*Fd*oversampling=8192 
%interpolated with Fast_Fading_Ts_factor, i.e. 819200 samples (each of duration Ts). This means that the first
%100samples of the chirp experiences the same fast fading.


noise_deviation_scale = 0.1; %corresponds to 10*log10(1/0.1^2) = 20dB (sigma = 0.1)
noise_samples = noise_samples*noise_deviation_scale; %+ noise_mean;    
    %scale the noise deviation to fit the scenario described above
figure(7)
stem([1:length(noise_samples)],real(noise_samples));
title('normalized noise samples for chirp signals')
ylabel('amplitude (real part)')
xlabel('noise sample number')

global Measurement_duration;
num_of_chirp_incr = Chirp_duration/Chirp_Ts;
num_of_chirps = (Measurement_duration/Chirp_duration);
chirp_signals = zeros(1,num_of_chirps*length(s_t));
system_outputs = zeros(1,num_of_chirps*length(s_t));
t = 1;

tau_rms = 0;
global Fast_Fading_Ts_factor;
h_no_doppler = h_g_t;
for k=1:num_of_chirps
    tau_avg = 0;
    tau_avg_pwr = 0;
    h = h_no_doppler;
    h_taps = find(abs(h) > (1/log(10+128+1))); 
        %used for independent fading sequences on each path,
        %do not consider sidelobes as seperate paths.
    for n=1:length(h_taps)
        tap = h(h_taps(n));
        %using each coefficient of doppler_seq_filtered for
        %Fast_Fading_Ts_factor number of consecutive samples of
        %distance Ts
        if(n==1) count = 1;
        elseif((h_taps(n)-h_taps(n-1))>2) count = count+1;
        end
        h(h_taps(n))=(doppler_seq_filtered(count,ceil(t/Fast_Fading_Ts_factor))).*tap;
            %modifying h according to fast fading
        
        h(h_taps(n))=h(h_taps(n)) + noise_samples(mod((t+n-1),length(noise_samples))+1);       
            %add independent noise to each path

        %coefficients for rms calculation
        tau_avg = tau_avg + (tap^2)*(h_taps(n)-h_taps(1));
        tau_avg_pwr = tau_avg_pwr + (tap^2)*(h_taps(n)-h_taps(1))^2;
    end
    tau_rms = tau_rms + (tau_avg_pwr/sum(h(h_taps).^2)) - ((tau_avg/sum(h(h_taps).^2)))^2;

 
    t=t+1;
    H_f = fft(h);
    
    %Important: The Simulation Output!
    system_outputs(((k-1)*num_of_chirp_incr + 1):(k*num_of_chirp_incr)) = (1/length(h))*ifft((S_t_freq.*fft(h).*S_t_inv_freq));
end

%tau_rms avaraged over the measurement period
tau_rms = sqrt(tau_rms/num_of_chirps);
disp(['RMS Delay spread for ' num2str(NPath) ' paths is: ' num2str(abs(tau_rms)) '*Ts (Ts = 10ns)'])

figure(8);
subplot(3,1,1);
plot([1:length(system_outputs(1:num_of_chirp_incr))],fftshift(abs(real(system_outputs(1:num_of_chirp_incr)))));
xlabel('system output with channel and filter, real part (first chirp)')
subplot(3,1,2)
plot([1:length(system_outputs(1:num_of_chirp_incr))],fftshift(abs(imag(system_outputs(1:num_of_chirp_incr)))));
xlabel('system output with channel and filter, imag part (first chirp)')
ylabel('amplitude response (abs)');
subplot(3,1,3)
plot([1:length(system_outputs(1:num_of_chirp_incr))],fftshift(abs((system_outputs(1:num_of_chirp_incr)))));
xlabel('system output with channel and filter, real + j*imag (first chirp)')


system_output_matrix = zeros(num_of_chirps,num_of_chirp_incr);
for n=1:num_of_chirps
    system_output_matrix(n,:) = (system_outputs(((n-1)*num_of_chirp_incr+1):(n*num_of_chirp_incr)));
end
figure(9)
surf((abs((system_output_matrix(:,[1:200])))));
zlabel('amplitude (abs)')
ylabel('measurement number')
xlabel('local time(n*Ts)')
title('Fast Fading. Reflected Components (absolute value of real+j*imag) versus Time')


end